
/**********************************************************************

  ASXXCFG.C -- AS52/AS76/AS96 modem driver setup utility.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Please note that the GPL allows you to use the driver, NOT the radio.
  In order to use the radio, you need a license from the communications
  authority of your country.

  Portions derived from YAMCFG/YAMDRV (by F1OAT/F6FBB/DG1KJD).

  History:
  0.1	   020127 DH1PA  FIRST OFFICIAL RELEASE
 **********************************************************************/

/* ------------------------------------------------------------------ */
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/errno.h>
#include <sys/socket.h>

#include <net/if.h>
#ifdef __GLIBC__
#include <netinet/if_ether.h>
#else
#include <linux/if_ether.h>
#endif

#include <asm/byteorder.h>

#include "asxx.h"
/* ------------------------------------------------------------------ */

#define is_mask(m, n) (m & n) ? '*' : ' '

static int set_params(char *name, struct ifreq *ifr, int sock, int ac, char **av, unsigned *pmask)
{
  int len;
  struct asxxdrv_ioctl_cfg asxxctl;

  printf("\n");

  asxxctl.cfg.mask = 0;

  while (ac >= 2)
  {
    len = strlen(av[0]);
      if (!strncasecmp(av[0], "iobase", len))
      {
        asxxctl.cfg.iobase = strtoul(av[1], NULL, 0);
        asxxctl.cfg.mask  |= ASXX_IOBASE;
      }
      else if (!strncasecmp(av[0], "irq", len))
      {
        asxxctl.cfg.irq   = strtoul(av[1], NULL, 0);
        asxxctl.cfg.mask |= ASXX_IRQ;
      }
      else if (!strncasecmp(av[0], "baudrate", len))
      {
        asxxctl.cfg.baudrate = strtoul(av[1], NULL, 0);
        asxxctl.cfg.mask    |= ASXX_BAUDRATE;
      }
      else if (!strncasecmp(av[0], "crctype", len))
      {
        asxxctl.cfg.crctype = strtoul(av[1], NULL, 0);
        asxxctl.cfg.mask   |= ASXX_CRCTYPE;
      }
      else if (!strncasecmp(av[0], "bitrate", len))
      {
        asxxctl.cfg.bitrate = strtoul(av[1], NULL, 0);
        asxxctl.cfg.mask   |= ASXX_BITRATE;
      }
      else if (!strncasecmp(av[0], "txdelay", len))
      {
        asxxctl.cfg.txdelay = strtoul(av[1], NULL, 0);
        asxxctl.cfg.mask   |= ASXX_TXDELAY;
      }
      else if (!strncasecmp(av[0], "txtail", len))
      {
        asxxctl.cfg.txtail = strtoul(av[1], NULL, 0);
        asxxctl.cfg.mask  |= ASXX_TXTAIL;
      }
      else if (!strncasecmp (av[0], "slottime", len))
      {
        asxxctl.cfg.slottime = strtoul (av[1], NULL, 0);
        asxxctl.cfg.mask |= ASXX_SLOTTIME;
      }
      else if (!strncasecmp (av[0], "persist", len))
      {
        asxxctl.cfg.ppersist = strtoul (av[1], NULL, 0);
        asxxctl.cfg.mask |= ASXX_PERSIST;
      }
      else
        fprintf(stderr, "%s: invalid parameter type '%s'\n", name, av[0]);

    av += 2;
    ac -= 2;
  }

  if (ac > 0)
    fprintf(stderr, "%s: orphaned parameter type '%s', no data\n", name, av[0]);

  if (asxxctl.cfg.mask)
  {
    ifr->ifr_data = (caddr_t) & asxxctl;
    asxxctl.cmd = ASXX_SETCFG;
    *pmask = asxxctl.cfg.mask;

      if (ioctl(sock, SIOCDEVPRIVATE, ifr) == -1)
      {
        fprintf(stderr, "%s: Error %s (%i), cannot ioctl ASXX_SETCFG on %s\n",
                name, strerror(errno), errno, ifr->ifr_name);
        return -1;
      }
  }
  return 0;
}

static void get_params(char *name, struct ifreq *ifr, int sock, unsigned mask)
{
  char *crctype;
  struct asxxdrv_ioctl_cfg asxxctl;

  ifr->ifr_data = (caddr_t) & asxxctl;
  asxxctl.cmd = ASXX_GETCFG;

  if (ioctl(sock, SIOCDEVPRIVATE, ifr) == -1)
  {
    fprintf(stderr, "%s: Error %s (%i), cannot ioctl ASXX_GETCFG on %s\n",
            name, strerror(errno), errno, ifr->ifr_name);
    return;
  }

  if (asxxctl.cfg.crctype)
    crctype = "RMNC";
  else
    crctype = "SMACK";

  printf("device:    %s\n",         ifr->ifr_name);
  printf("iobase:   %c%04x\n",      is_mask(mask, ASXX_IOBASE),   asxxctl.cfg.iobase);

  printf("irq:      %c%u\n",        is_mask(mask, ASXX_IRQ),      asxxctl.cfg.irq);
  printf("baudrate: %c%u\n",        is_mask(mask, ASXX_BAUDRATE), asxxctl.cfg.baudrate);
  printf("crctype:  %c%s\n",        is_mask(mask, ASXX_CRCTYPE),  crctype);

  printf("bitrate:  %c%u\n",        is_mask(mask, ASXX_BITRATE),  asxxctl.cfg.bitrate);
  printf("txdelay:  %c%-3u (ms)\n", is_mask(mask, ASXX_TXDELAY),  asxxctl.cfg.txdelay);
  printf("txtail:   %c%-3u (ms)\n", is_mask(mask, ASXX_TXTAIL),   asxxctl.cfg.txtail);

  printf ("slottime  :%c%-3u (ms)\n", is_mask (mask, ASXX_SLOTTIME), asxxctl.cfg.slottime);
  printf ("persist   :%c%u\n", is_mask (mask, ASXX_PERSIST), asxxctl.cfg.ppersist);
  printf("\n");
}

static const char *usage_str =
  "[-h] asxxif\n"
  "  [iobase <iobase>] [irq <irq>] [baudrate <baudrate>] [crctype <crctype>]\n"
  "  [bitrate <bitrate>] [txdelay <txdelay>] [txtail <txtail>]\n"
  "  not yet: [slottime <slottime>] [persist <persistence>]\n"
  "  asxxif: the name of the asxx kernel driver interface (asxx0)\n"
  "  -h: this help\n\n";

int main(int argc, char *argv[])
{
  char name[256] = "asxx0";
  struct ifreq ifr;
  int sock;
  unsigned int mask;

  printf("\n%s: Version 0.1 (C) 2002 by AATiS e.V.\n", *argv);

    if ((argc < 2) || (strcasecmp(argv[1], "-h") == 0))
    {
      printf("usage: %s %s", *argv, usage_str);
      return 1;
    }

  strcpy(name, argv[1]);
//    if ((sock = socket(PF_INET, SOCK_PACKET, htons(ETH_P_AX25))) < 0)
    if ((sock = socket(PF_PACKET, SOCK_DGRAM, htons(ETH_P_AX25))) < 0)
    {
      fprintf(stderr, "%s: Error %s (%i), cannot open %s\n",
              argv[0], strerror(errno), errno, name);
      return 2;
    }

  strcpy(ifr.ifr_name, name);
    if (ioctl(sock, SIOCGIFFLAGS, &ifr) < 0)
    {
      fprintf(stderr, "%s: Error %s (%i), cannot ioctl %s\n",
              argv[0], strerror(errno), errno, name);
      close(sock);
      return 3;
    }

  mask = 0;
    if (set_params(argv[0], &ifr, sock, argc-2, argv+2, &mask) == -1)
    {
      close(sock);
      return 4;
    }

  get_params(argv[0], &ifr, sock, mask);
  close(sock);
  return 0;
}
